#!/bin/sh                                                                                                                        
VERSION=1.0.0
ESXVER=$(uname -r)
#####################################
# eSilo Backup for VMWare ESXi
#
# Supported by:
# ESXi 3.x, 4.x, 5.x, and 6.x
#
# Tested on: 4.x, and 6.x
#
# This script utilizes the ghettoVCB.sh script from:
# https://communities.vmware.com/docs/DOC-8760
# Modified version from:
# https://github.com/lamw/ghettoVCB
# 
#####################################


####### BEGIN CONFIGURATION #########

# The directory in which the eSilo Backup scripts and VM backup files will
# be stored (backups under 'backups' subdirectory)
# This must be on a persistent datastore with enough free space for VM backups
ESILO_DIR=

# When files are larger than MAX_FILESIZE, they will be split into files of MAX_FILESIZE
# This must be 64GiB or less because eSilo Backup currently has a limit of 64GiB                                                                                  
MAX_FILESIZE=68719476736

# The eSilo Backup user ID to which backups are sent.
# Contact eSilo support if you do not know this value
ESILO_USER_ID=1234

# The eSilo Backup server host and port to connect to (leave at default unless you have a SiloServ)
ESILO_HOSTPORT=backup.esilo.com:1443

# The eSilo Backup schedule name
ESILO_SCHEDULENAME="VMWare ESXi v${ESXVER} Backup"

# The backup schedule in crontab format (default runs every day at 1 AM)
# min hour day mon dow
# NOTE: If you change this, you need to run 'esilobackup.sh postbootsetup' for changes to take effect.
# NOTE: The times here represent the UTC time on the VMWare ESXi server itself
CRON_SCHEDULE="0 1 * * *"

# Comma-separated list of email addresses to which the backup reports should be sent
ESILO_EMAILS=

# The list of VMs to be backed up (comma-separated and must be in quotes)
# Special value 'all' indicates to backup all VMs
VMS='Ubuntu 15.04
Windows Server 2008 R2'

####### END CONFIGURATION ###########


####### BEGIN FUNCTIONS #############


# Installs eSilo Backup and scripts to the specified path
install(){

	# Must have installation directory
	if [ -z "$ESILO_DIR" ] ; then
		echo "You must configure this script before running it"
		return 31
	fi
	
	# make install directory
	echo "Making directory '$ESILO_DIR' ..."
	mkdir -p "$ESILO_DIR"
	if [ $? -ne 0 ] ; then
		echo "Failed creating directory '$ESILO_DIR'; Can not continue."
		return 32
	fi
	

	# copy this script to the esilo folder
	echo "Copying this script to '$ESILO_DIR' ..."
	cp -f	 "$0" "$ESILO_DIR"
	RES=$?
	if [ $RES -ne 0 ] ; then
		echo "Failed to copy script '$0' to '$ESILO_DIR'; Can not continue."
		return 41
	fi

	echo "Changing directory to '$ESILO_DIR' ..."
	cd "$ESILO_DIR"
	if [ $? -ne 0 ] ; then
		echo "Failed changing directory to '$ESILO_DIR'; Can not continue."
		return 33
	fi

	echo "Downloading eSilo Backup binary..."
	TARBALL="$(basename "$ESILOBACKUP_URL")"
	if [ -e "$TARBALL" ] ; then
		rm -f "$TARBALL"
	fi
	wget "$ESILOBACKUP_URL"
	if [ $? -ne 0 ] ; then
		echo "Failed downloading eSilo Backup from '$ESILOBACKUP_URL' to '$ESILO_DIR'; Can not continue."
		return 34
	fi
	
	echo "Extracting '$TARBALL' ..."
	tar xzvf "$TARBALL"
	if [ $? -ne 0 ] ; then
		echo "Failed extracting '$TARBALL' to '$ESILO_DIR'; Can not continue."
		return 35
	fi

	echo "Making esilobackup executable..."
	chmod +x "esilobackup"
	RES=$?
	if [ $RES -ne 0 ] ; then
		echo "Failed making 'esilobackup' executable (code $RES); Can not continue."
		return 36
	fi

	echo "Making split executable..."
	chmod +x "split"
	RES=$?
	if [ $RES -ne 0 ] ; then
		echo "Failed making 'split' executable (code $RES); Can not continue."
		return 37
	fi

	echo "Making ghettoVCB.sh executable..."
	chmod +x "ghettoVCB.sh"
	RES=$?
	if [ $RES -ne 0 ] ; then
		echo "Failed making 'ghettoVCB.sh' executable (code $RES); Can not continue."
		return 41
	fi


	echo "Setting eSilo Backup host and port..."
	HOSTSFILE="$ESILO_DIR/etc/hosts"
	echo "$ESILO_HOSTPORT" > "$HOSTSFILE" 
	if [ $? -ne 0 ] ; then
		echo "Failed saving eSilo backup server host and port '$ESILO_HOSTPORT' to '$HOSTSFILE'; Can not continue."
		return 38
	fi
	
	echo "Updating RC script..."
	MANUAL=1
	grep -q "esilobackup.sh" "$RCLOCAL"
	if [ $? -eq 0 ] ; then
		MANUAL=1
	else

		# remove exit 0 at the end of the script if it exists
		COUNT=$(grep -c "^ *exit *0 *$" "$RCLOCAL")
		if [ $COUNT -gt 0 ] ; then
		
			# If the only instance of 'exit 0' is the last line, we can safely remove it,
			# otherwise, indicate to user than manual editing will be required
			if [ $COUNT -eq 1 ] ; then
				tail -n 100 "$RCLOCAL" | grep '^.'  | tail -n 1 | grep "^ *exit *0 *$"
				if [ $? -eq 0 ] ; then
					sed -i "s/^ *exit *0 *$//" "$RCLOCAL"
					MANUAL=0
				else
					MANUAL=1
				fi
			else
				MANUAL=1
			fi
		else
			if [ $COUNT -eq 0 ] ; then
				MANUAL=0
			fi
		fi
	fi

	if [ $MANUAL -ne 0 ] ; then

		echo ""
		echo "NOTICE: RC script '$RCLOCAL' seems to already have an entry for the eSilo Backup post-boot setup."
		echo "Please verify this to ensure that eSilo will run across reboots"
		echo ""

	else
		
		echo -e "${EXEC_PREFIX}\"$ESILO_DIR/esilobackup.sh\" postbootsetup\n\nexit 0" >> "$RCLOCAL"
		RES=$?
		if [ $RES -ne 0 ] ; then
			echo "Failed appending to startup script '$RCLOCAL'; Can not continue."
			return 39
		fi

	fi
	
	echo "Running post boot setup..."
	postbootsetup
	if [ $? -ne 0 ] ; then
		echo "Post boot setup failed; Can not continue."
		return 40
	fi
	
	
	echo "Installation completed. You have one manual step to complete:"
	echo "* Save your eSilo Backup encryption key to: '$ESILO_DIR/etc/account.key'"
	echo ""
	return 0

}

# Some ESXi settings are lost across reboots
# This function makes those configuration changes
postbootsetup(){

	# Create OpenSSL symlink (esilobackup expects libcrypto.so.4/6/etc, depending on build)
	echo "Creating OpenSSL symlink..."
	OPENSSL_LIB="$LIBDIR/${OPENSSLVER}"
	if [ ! -e "$OPENSSL_LIB" ] ; then
		echo "OpenSSL library '$OPENSSL_LIB' not found; Can not continue."
		return 20
	fi
	OPENSSL_SYMLINK="$LIBDIR/${OPENSSLSYM}"
	ln -s "$OPENSSL_LIB" "$OPENSSL_SYMLINK" 2>/dev/null
	RES=$?
	if [ $RES -ne 0 -a $RES -ne 1 ] ; then
		echo "Failed creating OpenSSL symlink '$OPENSSL_SYMLINK' -> '$OPENSSL_LIB' (exit code $RES); Can not continue."
		return 21
	fi

	# Stop cron service so we can update the crontab file
	echo "Stopping cron service..."
	/bin/kill $(cat /var/run/crond.pid)
	if [ $? -ne 0 ] ; then
		echo "Failed executing '$KILLCRONCMD' to stop cron service; Can not continue."
		return 22
	fi

	# Add eSilo Backup entry to cron file
	grep -q "esilobackup" "$CRONTAB_FILE"
	if [ $? -eq 0 ] ; then
		echo ""
		echo "NOTICE: There seems to already be an esilo backup entry in the crontab file '$CRONTAB_FILE'; Not adding another."
		echo "        If you want multiple entries, you will have to add them yourself manually."
		echo ""
	else
		echo "Updating crontab file '$CRONTAB_FILE' ..."
		echo "$CRON_SCHEDULE ${EXEC_PREFIX}\"$ESILO_DIR/esilobackup.sh\" backup > \"${ESILO_DIR}/esilobackup.log\" 2>&1" >> "$CRONTAB_FILE"
		if [ $? -ne 0 ] ; then
			echo "Failed writing crontab entry to '$CRONTAB_FILE'; Can not continue."
			return 23
		fi
	fi

	# Start cron
	echo "Starting cron service..."
	$STARTCRONCMD
	RES=$?
	if [ $RES -ne 0 ] ; then
		echo "Failed executing '$STARTCRONCMD' to start cron service (exit code $RES). Can not continue."
		return 24
	fi


	# ESXi 6.x only: add firewall rule
	if [ "$ESXVER" == "6.0.0" ] ; then
		echo "Adding firewall rule..."

		# Add firewall rule to firewall config file	(if it does not already exist)
		grep -q 'eSilo Backup Out'  "$FIREWALL_XML_FILE"
		if [ $? -ne 0 ] ; then
		
			LASTID=$(grep 'service id *='  "$FIREWALL_XML_FILE" | grep -o '[0-9]\+' | sed 's/0\+\([0-9]\+\)/\1/' | sort -n | tail -n 1)
			NEXTID=$(($LASTID+1))
			IDSTR=$(printf '%04d' $NEXTID)
			SEDCMD='s/<\/ConfigRoot>/  <service id="'$IDSTR'">   \n    <id>eSilo Backup Out<\/id>\n    <rule id="0000">                     \n    <direction>outbound<\/direction>\n    <protocol>tcp<\/protocol>\n    <porttype>dst<\/porttype>\n    <port>1443<\/port>\n    <\/rule>\n    <enabled>true<\/enabled>\n    <required>false<\/required>\n  <\/service>\n\n<\/ConfigRoot>/' 
	
			sed -i "$SEDCMD" "$FIREWALL_XML_FILE"
			RES=$?
			if [ $RES -ne 0 ] ; then
				echo "Failed adding eSilo Backup firewall rule	to '$FIREWALL_XML_FILE'; Can not continue."
				return 27
			fi

			echo "Restarting firewall..."
			esxcli network firewall refresh
			RES=$?
			if [ $? -ne 0 ] ; then
				echo "Failed restarting firewall; Can not continue."
				return 25
			fi



		fi
	fi					

	# Create symlink to /usr/local/esilo (esilobackup fails without this)
	mkdir -p /usr/local                                                                                                                                 
	if [ ! -e /usr/local/esilo ] ; then                                                                                              
		ln -s "$ESILO_DIR" /usr/local/esilo
		RES=$?
		if [ $RES -ne 0 ] ; then
			echo "Failed creating symlink '/usr/local/esilo' -> '$ESILO_DIR'; Can not continue."
			return 26
		fi
	fi                                                                                                                               
	

	echo "Post-Boot Setup Completed."
	return 0

}

# Runs the VM backup and eSilo Backup
backup(){

	# Some sanity checks
	if [ ! -e "$ESILO_DIR/etc/account.key" ] ; then
		echo "Fatal: The eSilo Backup encryption key file '$ESILO_DIR/etc/account.key' does not exist; Can not continue."
		return 10
	fi

	if [ -z "$VMS" ] ; then
		echo "Fatal: No virtual machines are configured to be backed up; Can not continue."
		return 11
	fi

	# Create vms_to_backup file

	VMS_FILE="$ESILO_DIR/vms_to_backup"
	>	"$VMS_FILE"
	for VM in "$VMS" ; do
		echo "$VM" >> "$VMS_FILE"
		if [ $? -ne 0 ] ; then
			echo "Failed to write VM list file '$VMS_FILE'; Can not continue."
			return 12
		fi
	done

	# Set VM_BACKUP_VOLUME in ghettovcb.conf
	echo "Setting VM_BACKUP_VOLUME ..."
	GVCB_CONF="$ESILO_DIR/ghettoVCB.conf"
	sed -i "s:^VM_BACKUP_VOLUME=.*:VM_BACKUP_VOLUME=${ESILO_DIR}/backups:" "$GVCB_CONF"
	RES=$?
	if [ $RES -ne 0 ] ; then
		echo "Failed updating VM_BACKUP_VOLUME in '$GVCB_CONF' (exit code $RES); Can not continue."
		return 13
	fi

	# Set VM_BACKUP_NAMING_CONVENTION in ghettovcb.sh
	echo "Setting VM_BACKUP_DIR_NAMING_CONVENTION..."	
	grep -q '^VM_BACKUP_NAMING_CONVENTION=' "$GVCB_CONF"
	if [ $? -eq 0 ] ; then
		sed -i 's/^VM_BACKUP_DIR_NAMING_CONVENTION=.*/VM_BACKUP_DIR_NAMING_CONVENTION="esilo"/' "$GVCB_CONF"
		RES=$?
		if [ $RES -ne 0 ] ; then
			echo "Failed updating VM_BACKUP_VOLUME in '$GVCB_CONF' (exit code $RES); Can not continue."
			return 17
		fi
	else
		echo "VM_BACKUP_DIR_NAMING_CONVENTION=esilo" >> "$GVCB_CONF"
	fi


	# Write eSilo Backup schedule file
	echo "Writing eSilo Backup schedule file..."
	ESILO_SCHEDULE_FILE="$ESILO_DIR/etc/schedule"
	echo -e "name=$ESILO_SCHEDULENAME\nuserid=$ESILO_USER_ID\ndest=/$ESILO_USER_ID/Today\nsrc=$ESILO_DIR\nxsrc=$ESILO_DIR/#FCISORT#\nemail=$ESILO_EMAILS" > "$ESILO_SCHEDULE_FILE"
	if [ $? -ne 0 ] ; then
		echo "Failed writing eSilo Backup schedule file '$ESILO_SCHEDULE_FILE'; Can not continue."
		return 14
	fi



	# Create backups directory
	mkdir -p "$ESILO_DIR/backups"                                                                                                     

	# Remove old backup files
	echo Removing old backups...                                                                                                     
	rm -rf "$ESILO_DIR/backups/"*                                                                                                     
	echo "Done ($?)"                                                                                                                 

	# execute ghettoVCB
	echo Running ghettoVCB...                                                                                                        
	"$ESILO_DIR/ghettoVCB.sh"  -f "$ESILO_DIR/vms_to_backup"  -g "$ESILO_DIR/ghettoVCB.conf" -l "$ESILO_DIR/ghettoVCB.log"               
	RES=$?
	if [ $RES -ne 0 ] ; then
		echo "ghettoVCB failed with exit code $RES; Can not continue."
		return 16
	fi
# TODO: if only some VMs failed, still continue

	# Split files larger than $MAX_FILESIZE 
	while read FILE ; do
	                                                                                                                                 
	        BF="$(basename "$FILE")"                                                                                                 
	        DIR="$(dirname "$FILE")"                                                                                                 
	        cd "$DIR"                                                                                                                

					# split
	        echo "Splitting $FILE ... "                                                                                              
	        "$ESILO_DIR/split" "$BF" "${BF}." -b $MAX_FILESIZE -d                                                                         
	                                                                                                                                 
	        # remove original file                                                                                                   
	        rm -f "$FILE"                                                                                                            
	                                                                                                                                 
	done<<HEREDOC
$(find "$ESILO_DIR/backups/" -type f -size +${MAX_FILESIZE}c)           
HEREDOC

	echo "Done; Running esilo backup..."                                                                                             
	"$ESILO_DIR/esilobackup" "$ESILO_DIR/etc/schedule"                                                                                 
	RES=$?
	if [ $RES -ne 0 ] ; then
		echo "Failed executing eSilo Backup (exit code $RES); Can not continue."
		return 15
	fi

	echo "Completed successfully"
	return 0
}

usage(){

	echo "eSilo Backup for VMWare EXSi v${VERSION}"
	echo "Usage:"
	echo "esilobackup.sh [ install | postbootsetup | backup ]"
	echo ""
	echo "install        Installs eSilo Backup and makes necessary configuration changes"
	echo "postbootsetup  Makes configuration changes required after reboot"
	echo "backup         Runs the VM backup script and then eSilo Backup"
	echo ""

}

####### END FUNCTIONS ###############






####### BEGIN MAIN SCRIPT ###########



if [ -z "$ESILO_DIR" ] ; then
	echo "You must configure this script before using it."
	exit 99
fi

# Variables common to all ESXi versions
CRONTAB_FILE="/var/spool/cron/crontabs/root"
FIREWALL_XML_FILE="/etc/vmware/firewall/service.xml"


case "$ESXVER" in

	"6.0.0")
	
		ESILOBACKUP_URL=http://downloads.esilo.com/vmware/esilobackup-esxi6.tgz
		RCLOCAL=/etc/rc.local.d/local.sh
		LIBDIR=/lib64
		OPENSSLVER="libcrypto.so.1.0.1"
		OPENSSLSYM="libcrypto.so.4"
		STARTCRONCMD="crond"
		EXEC_PREFIX=""
		
	;;
	
	"4.0.0")
	
		ESILOBACKUP_URL=http://downloads.esilo.com/vmware/esilobackup-esxi4.tgz
		RCLOCAL=/etc/rc.local
		LIBDIR=/lib
		OPENSSLVER="libcrypto.so.0.9.8"
		OPENSSLSYM="libcrypto.so.4"
		STARTCRONCMD="/bin/busybox crond"
		EXEC_PREFIX="/bin/sh "
		
	;;
	
	*)
	
		echo "Your VMWare ESXi version: '$ESXVER' supported: No. Please contact eSilo Support."
		exit 1
		
	;;
	
esac

echo "Your VMWare ESXi version: '$ESXVER' supported: Yes"
echo ""




case "$1" in 

	"install")
	
		install "$2"
		exit $?

	;;

	"postbootsetup")
	
		postbootsetup
		exit $?

	;;
	
	"backup")
	
		backup
		exit $?

	;;
	
	*)
	
		usage
		exit $?

	;;

esac
	


